require 'fileutils'

require 'razor/view'

module Razor

	class Generable
		
		attr_reader :parent, :src_name
		
		def initialize(parent, src_name)
			@parent = parent
			@src_name = src_name
		end
		
		alias dest_name src_name
		
		def src
			@src ||= File.join(parent.src, src_name)
		end
	
		def dest
			@dest ||= File.join(parent.dest, dest_name)
		end
		
		def url
			@url ||= File.join(parent.url, dest_name)
		end
		
		def http(url)
			url.empty? and return self
			return nil
		end
		
	end
	
	class GenerableFile < Generable

		def contents
			File.read(src, :mode=>'rb')
		end

		def generate
			File.open(dest, 'wb') { |f| f << contents }
		end
		
		def request
			self
		end
		
		def dest_ext
			File.extname(dest_name)
		end
	end
	
	class Directory < Generable
		
		attr_reader :files
		
		def initialize(parent, src_name)
			super(parent, src_name)
			@files = Dir.entries(src).reject { |filename|
				filename =~ /^[\._].*/ or filename=='Rakefile'
			}.map { |filename|
				if File.directory? File.join(src,filename)
					Directory.new(self, filename)
				elsif filename =~ /^[^~].*\.rb$/
					ViewFile.new(self, filename)
				else
					RegularFile.new(self, filename)
				end
			}
		end
	
		def generate
			Dir.mkdir(dest) rescue nil
			@files.each(&:generate)
		end
		
		def http(url)
			s = super(url)
			s and return s
			name, *rest = url.split('/')
			file = @files.find { |f| f.dest_name == name }
			file or return nil
			file.http(rest*'/')
		end
		
		def request
			@files.find { |f| f.dest_name == 'index.html' }
		end
	end

	class Site < Directory

		attr_reader :src, :dest, :url
		def initialize(src, dest, url_base='/')
			View.reset_instances!
			@src, @dest, @url = src, dest, url_base
			View.layouts_path = File.join(src, '_layouts/')
			View.url_base = url
			 Razor.load '_init.rb' rescue nil
		rescue LoadError
			$!.message =~ /^no such file to load -- (.+)$/
			puts "Can't load #{$1} file"
		ensure
			super(nil, nil)
		end
		
	end

	class RegularFile < GenerableFile
		
		def dest_name
			# delete first '~' if needed.
			super =~ /([^~].+)/
			return $1
		end
		
		def generate
			FileUtils.cp(src, dest)
		end
	end
	
	class ViewFile < GenerableFile
		
		def initialize(parent, src_name)
			super(parent, src_name)
			@view = View.eval(self)
		end
		
		def dest_name
			ext = @view.class.get_extension
			src_name.sub(/rb$/, ext)
		end
		
		def contents
			@view.render
		end
			
	end

end
