require 'mustache'
require 'razor/renderer'

module Razor

	class View
		
		class << self
			
			attr_reader :definition, :playout, :instances
			
			def inherited(subclass)
				@subclasses << subclass
				subclass.class_initialize
			end
			
			def class_initialize
				@subclasses = []
				@definition = Class.new(superclass.definition)
				@playout = superclass.playout
				@fields = {}
				@instances = []
			end
			
			def reset_instances!
				@instances = []
				@subclasses.each(&:reset_instances!)
			end
			
			def eval(viewfile)
				@@last_evaluated = nil
				@@last_viewfile = viewfile
				Kernel.eval(File.read(viewfile.src), TOPLEVEL_BINDING)
				return @@last_evaluated
			end
			
			def extension(ext)
				@extension = ext
			end
			
			def layouts_path=(path)
				@@layouts_path = path
			end
			
			def url_base
				@@url_base
			end
		
			def url_base=(url_base)
				@@url_base = url_base
			end
		
			def get_extension
				if superclass.respond_to? :get_extension
					@extension or superclass.get_extension
				else
					@extension
				end
			end
			
			def layout(str=nil, &block)
				@playout = str ? lambda { str } : block
			end
			
			def field(name, default=nil)
				if name =~ /^(field|contents)$/
					raise "Try to create a #{name.inspect} field"
				end
				attr_accessor name
				@fields["@#{name}"] = default
				@definition.field(name)
			end
			
			def fields
				if superclass.respond_to? :fields
					superclass.fields.merge(@fields)
				else
					@fields
				end
			end
			
			def new(*args, &block)
				instance = super(*args, &block)
				@instances << instance
				@@last_evaluated = instance
				return instance
			end
			
			def all
				@subclasses.inject(@instances) { |ary, subclass|
					ary + subclass.instances
				}
			end
			
			def layout_render(instance)
				render(Mustache::Context.new(instance))
			end
			
			def render(context)
				renderer = Renderer.get(@@layouts_path, @playout)
				if superclass.respond_to? :render
					if renderer.valid?
						context.push('yield' => renderer.layout_render(context))
					end
					superclass.render(context)
				else
					if renderer.valid?
						renderer.layout_render(context)
					else
						context['yield']
					end
				end
			end
			
		end
			
		@extension = ''
		@playout = nil
		@fields = {}
		@instances = []
		@subclasses = []
		
		attr_accessor :contents
		
		def initialize(*args, &block)
			@viewfile = @@last_viewfile
			@contents = lambda { '' }
			self.class.fields.each { |iv|
				instance_variable_set(*iv)
			}
			if block_given?
				self.class.definition.new(self, &block)
			end
		end
		
		def url
			@viewfile.url
		end
		
		def url_base
			self.class.url_base
		end
		
		def field(name, value)
			(class<<self; self; end).class_eval { attr name }
			instance_variable_set("@#{name}", value)
		end
		
		def yield
			Renderer.get(@viewfile.parent.src, @contents).render(self)
		end
		
		def render
			self.class.layout_render(self)
		end
		
		@definition = Class.new {
			
			def self.field(name)
				class_eval(<<-END)
					def #{name}(val)
						@view.#{name} = val
					end
				END
			end
			
			def initialize(view, &block)
				@view = view
				instance_exec &block
			end
			
			def field(name, value)
				@view.field(name, value)
			end
			
			def contents(str=nil, &block)
				@view.contents = str ? lambda { str } : block
			end
			
			def view_eval(&block)
				@view.instance_eval(&block)
			end
	
		}
		
	end
	
	class Page < View
		
		extension 'html'
		
		layout { from_file 'page.html' }
		
	end
	
	class StyleSheet < View
		
		extension 'css'
		
		layout { from_file 'stylesheet.css' }
		
	end
	
	class Script < View
		
		extension 'js'
		
		layout { from_file 'script.js' }
		
	end
	
end